var ExtractUtils = function()
{
	var exutils = {};
	var debug = 0;

	exutils.getQiacubeSortedPrefixes = function(primaryQiacubePrefix, secondaryQiacubePrefix)
	{
		// Default value for primary QIAcube prefix is "B", and for secondary is "C"
		if (primaryQiacubePrefix == null || primaryQiacubePrefix == '')
		{
			primaryQiacubePrefix = 'B';
		}
		if (secondaryQiacubePrefix == null || secondaryQiacubePrefix == '')
		{
			secondaryQiacubePrefix = 'C';
		}

		// Get string of current QIAcube prefixes in order
		var fullSortedPrefixSet = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';
		var sortedPrefixes = '' + primaryQiacubePrefix + secondaryQiacubePrefix;
		for (var i = 0; i < fullSortedPrefixSet.length; i++)
		{
			// Get next prefix
			var prefix = fullSortedPrefixSet.substring(i, i+1);
			// Add prefix to string of sorted prefixes, if not already present
			if (prefix != primaryQiacubePrefix && prefix != secondaryQiacubePrefix)
			{
				sortedPrefixes += prefix;
			}
		}
		return sortedPrefixes;
	}

	/**
		Returns a QIAcube position string with optional instrument
		letter prefix A-Z for vial 'i', when 'nitems' vials are
		to be analyzed on at most two instruments.

		If the number of items is <= 12, all will be placed in
		the QIAcube with the primary prefix. If the number of items
		is > 12 and <= 24, they will be distributed between
		two instruments as evenly as possible, except that one
		tries to avoid having 11 vials in the instrument with
		most vials, as this would require a counterbalance. For 23
		vials the instruments with less vials (11) will need a
		counterbalance. 
	 */
	exutils.getDetailedQiaCubePosition = function(nitems, i, qiacubeSortedPrefixes)
	{
		var dpos = null;
		var qiacubePos = null;
		// Two QIAcube instruments are available, default is 'B' and 'C'
		var qiacubeId = qiacubeSortedPrefixes.substring(0, 1);
		if (nitems == null || nitems < 1 || nitems > 24)
		{
			return null;
		}
		if (i == null || i < 1 || i > nitems)
		{
			return null;
		}
		if (nitems <= 12)
		{
			qiacubePos = exutils.getQiaCubePosition(nitems, i);
		}
		else
		{
			var numItemsQiaCube1 = Math.floor((nitems + 1)/2);
			if (numItemsQiaCube1 == 11)
			{
				numItemsQiaCube1 = 12;
			}
			if (i <= numItemsQiaCube1)
			{
				// Place sample in first QIAcube
				qiacubePos = exutils.getQiaCubePosition(numItemsQiaCube1, i);
			}
			else
			{
				// Place sample in second QIAcube
				var numItemsQiaCube2 = nitems - numItemsQiaCube1;
				qiacubePos = exutils.getQiaCubePosition(numItemsQiaCube2, i - numItemsQiaCube1);
				qiacubeId = qiacubeSortedPrefixes.substring(1, 2);
			}
		}
		if (qiacubeId != null && qiacubePos != null)
		{
			dpos = qiacubeId + qiacubePos;
		}
		return dpos;
	}

	/**
		Returns a QIAcube position number in a single instrument
		for vial 'i', when 'nitems' vials are placed in the instrument.
		The numbers are based on recommendations from the manufacturer,
		in order to ensure balance stability of the centrifuge.
		For a total of 1 or 11 items, it is assumed that a counterbalance
		in the proper positions is used.
	 */
	exutils.getQiaCubePosition = function(nitems, i)
	{
		var pos = null;
		if (nitems == null || nitems < 1 || nitems > 12)
		{
			return null;
		}
		if (i == null || i < 1 || i > nitems)
		{
			return null;
		}
		var posArr = ['1','2','3','4','5','6','7','8','9','10','11','12'];
		if (nitems == 2)
		{
			posArr = ['1','7','','','','','','','','','',''];
		}
		else if (nitems == 3)
		{
			posArr = ['1','5','9','','','','','','','','',''];
		}
		else if (nitems == 4)
		{
			posArr = ['1','2','7','8','','','','','','','',''];
		}
		else if (nitems == 5)
		{
			posArr = ['1','2','5','8','9','','','','','','',''];
		}
		else if (nitems == 6)
		{
			posArr = ['1','2','3','7','8','9','','','','','',''];
		}
		else if (nitems == 7)
		{
			posArr = ['1','2','3','6','7','9','10','','','','',''];
		}
		else if (nitems == 8)
		{
			posArr = ['1','2','3','4','7','8','9','10','','','',''];
		}
		else if (nitems == 9)
		{
			posArr = ['1','2','3','5','6','7','9','10','11','','',''];
		}
		else if (nitems == 10)
		{
			posArr = ['1','2','3','4','5','7','8','9','10','11','',''];
		}
		else if (nitems == 11)
		{
			// Counterbalance blank vial filled with water needed i position 12.
			posArr = ['1','2','3','4','5','6','7','8','9','10','11',''];
		}
		pos = posArr[i-1];
		return pos;
	}

	/**
		Returns a QIAcube position string with optional instrument
		letter prefix A-Z from a pure integer position value.

		Examples for qiacubeSortedPrefixes = 'BCADEFGHIJKLMNOPQRSTUVWXYZ' ("B" is default)
		intPosition (  1 -  12) => positionStr = 'B' + integer
		intPosition ( 13 -  24) => positionStr = 'C' + (integer - 12)
		intPosition (301 - 312) => positionStr = 'Z' + (integer - 300)
	 */
	exutils.getDetailedQiacubePosition = function(intPosition, qiacubeSortedPrefixes)
	{
		var qiacubePositionStr = null;
		if (intPosition && intPosition > 0)
		{
			var numPrefixes = qiacubeSortedPrefixes.length;
			var prefixIndex = Math.floor(intPosition/12);
			if (prefixIndex < numPrefixes)
			{
				var qiacubePrefix = qiacubeSortedPrefixes.substring(prefixIndex, prefixIndex + 1);
				var qiacubePos = intPosition - 12*prefixIndex;
				qiacubePositionStr = qiacubePrefix + qiacubePos;
			}
			else
			{
				qiacubePositionStr = '' + intPosition;
			}
		}
		return qiacubePositionStr;
	}

	/**
		Returns a pure integer position value from a QIAcube position
		string with optional instrument letter prefix A-Z.
		Returns `null` if input position string is `null`.

		positionStr = integer          => QIAcube position = integer (1-12)
		positionStr = Prefix + integer => QIAcube position = integer + 12*<Prefix order number 0-25>
	 */
	exutils.getIntQiacubePosition = function(positionStr, qiacubeSortedPrefixes)
	{
		var intPos = null;
		if (positionStr)
		{
			// Check if string contains uppercase letter + QIAcube position number 1-12
			if (/^[A-Z]([2-9]|1[0-2]?)$/.test(positionStr))
			{
				var qiaCubePrefix = positionStr.substring(0,1);
				// Get QIAcube prefix order number 0-25
				var qiacubePrefixOrder = qiacubeSortedPrefixes.indexOf(qiaCubePrefix);
				if (qiacubePrefixOrder >= 0)
				{
					intPos = parseInt(positionStr.substring(1)) + 12*qiacubePrefixOrder;
				}
			}
			if (intPos == null)
			{
				intPos = parseInt(positionStr);
			}
		}
		return intPos;
	}

	exutils.isMeludiItem = function(item)
	{
		var isMeludiItem = false;
		if (item != null)
		{
			var name = item.name;
			if (name != null)
			{
				// Check that item name starts with "ML" to exclude extra items
				if (name.indexOf('ML') == 0)
				{
					isMeludiItem = true;
				}
			}
		}
		return isMeludiItem;
	}

	exutils.isExtraMaterial = function(item)
	{
		var isExtraMaterial = false;
		if (item != null)
		{
			var name = item.name;
			if (name != null)
			{
				// Check that item name does not start with "ML" to exclude MeLuDI items
				if (name.indexOf('ML') != 0)
				{
					isExtraMaterial = true;
				}
			}
		}
		return isExtraMaterial;
	}

	exutils.isSpecimen = function(item)
	{
		var isSpecimen = false;
		if (item != null)
		{
			var name = item.name;
			if (name != null)
			{
				// Check that item name ends with single dot "." followed by a natural number (>0)
				// "\x2E" represents a dot "." (2E is the hexadecimal ASCII code for ".")
				var pattern = /^[A-Za-z]+[A-Za-z0-9_-]*\x2E[1-9]+[0-9]*$/
				if (pattern.test(name))
				{
					isSpecimen = true;
				}
			}
		}
		return isSpecimen;
	}

	exutils.isRna = function(item)
	{
		var isRna = false;
		if (item != null)
		{
			var name = item.name;
			if (name != null)
			{
				// Check if item name ends with ".r"
				if (name.indexOf('.r') == (name.length - 2))
				{
					isRna = true;
				}
			}
		}
		return isRna;
	}

	exutils.isDna = function(item)
	{
		var isDna = false;
		if (item != null)
		{
			var name = item.name;
			if (name != null)
			{
				// Check if item name ends with ".d"
				if (name.indexOf('.d') == (name.length - 2))
				{
					isDna = true;
				}
			}
		}
		return isDna;
	}

	exutils.findSeparator = function(data, searchString)
	{
		// Searches for the first line containing the input search string, then returns next character as separator
		var separator = null;
		if (data != null && searchString != null)
		{
			var lines = data.split(/[\n\r]+/);
			var found = false;
			for (var i = 0; (i < lines.length && !found); i++)
			{
				if (lines[i].match(/^\s*$/)) continue; // Ignore empty lines
				var line = lines[i];
				var index = line.indexOf(searchString);
				if (index >= 0)
				{
					// Get next character after first column header as separator
					var separatorIndex = index + searchString.length;
					separator = line.substring(separatorIndex, separatorIndex+1);
					found = true;
				}
			}
		}
		return separator;
	}

	exutils.setDecimalDotInsideDoubleQuotes = function(line)
	{
		var lineParts = line.split(/ *" */);
		var procLine = lineParts[0];
		var numAddedDblQts = 0;
		for (var i = 1; i < lineParts.length; i++)
		{
			var str = lineParts[i];
			if (i % 2 == 1)
			{
				// Inside double quotes
				// Replace decimal comma with decimal dot
				str = str.replace(/,/, '.');
			}
			// Add string to processed line
			procLine += '"' + str;
			numAddedDblQts++;
		}
		if (numAddedDblQts % 2 == 1)
		{
			// Add terminal double quote			
			procLine += '"';
		}
		// Return processed line
		return procLine;
	}

	exutils.getNumber = function(sValue)
	{
		return parseFloat(sValue.replace(/,/, '.'));
	}

	return exutils;
}();

//Doc.onLoad(ExtractUtils.initPage);

